/*******************************************************************************

Copyright (c) 1997-2006, Perforce Software, Inc.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1.  Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

2.  Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTR
IBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL PERFORCE SOFTWARE, INC. BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

/*******************************************************************************
 * Name		: p4mergedata.cc
 *
 * Author	: Tony Smith <tony@perforce.com> or <tony@smee.org>
 *
 * Description	: Class for holding merge data
 *
 ******************************************************************************/
#include <ruby.h>
#include "undefdups.h"
#include <clientapi.h>
#include <i18napi.h>
#include <strtable.h>
#include <spec.h>
#include "extconf.h"
#include "p4result.h"
#include "p4rubydebug.h"
#include "clientuserruby.h"
#include "p4mergedata.h"


static void mergedata_free( P4MergeData *md )
{
    delete md;
}

static void mergedata_mark( P4MergeData *md )
{
    md->GCMark();
}


P4MergeData::P4MergeData( ClientUser *ui, ClientMerge *m, StrPtr &hint )
{
    this->debug = 0;
    this->ui = ui;
    this->merger = m;
    this->hint = hint;

    // Extract (forcibly) the paths from the RPC buffer.
    StrPtr *t;
    if( ( t = ui->varList->GetVar( "baseName" ) ) ) base = t->Text();
    if( ( t = ui->varList->GetVar( "yourName" ) ) ) yours = t->Text();
    if( ( t = ui->varList->GetVar( "theirName" ) ) ) theirs = t->Text();
    
}

VALUE
P4MergeData::GetYourName()
{
    return rb_str_new2( yours.Text() );
}

VALUE
P4MergeData::GetTheirName()
{
    return rb_str_new2( theirs.Text() );
}

VALUE
P4MergeData::GetBaseName()
{
    return rb_str_new2( base.Text() );
}


VALUE
P4MergeData::GetYourPath()
{
    return rb_str_new2( merger->GetYourFile()->Name() );
}

VALUE
P4MergeData::GetTheirPath()
{
    return rb_str_new2( merger->GetTheirFile()->Name() );
}

VALUE
P4MergeData::GetBasePath()
{
    return rb_str_new2( merger->GetBaseFile()->Name() );
}

VALUE
P4MergeData::GetResultPath()
{
    return rb_str_new2( merger->GetResultFile()->Name() );
}

VALUE
P4MergeData::GetMergeHint()
{
    return rb_str_new2( hint.Text() );
}

VALUE
P4MergeData::RunMergeTool()
{
    Error e;
    ui->Merge( merger->GetBaseFile(), merger->GetTheirFile(),
	       merger->GetYourFile(), merger->GetResultFile(), &e );

    if( e.Test() )
	return Qfalse;
    return Qtrue;
}


VALUE
P4MergeData::Wrap( VALUE pClass )
{
    VALUE md;
    VALUE argv[ 1 ];

    md = Data_Wrap_Struct( pClass, mergedata_mark, mergedata_free, this );
    rb_obj_call_init( md, 0, argv );
    return md;
}

void
P4MergeData::GCMark()
{
    // We don't hold Ruby objects
}

